// Receive Sony IR Remote signals using PIC16F84A
// external crystal 8MHz , watchdog disabled
// connect 3-pin IR receiver module to PORTA.2 input
// Sony TV remote will toggle:
//     PORTB 0 output for TV '1' button
//     PORTB 1 output for TV '2' button
//     PORTB 2 output for TV '3' button
//     PORTB 3 output for TV '4' button
//     PORTB 4 output for TV '5' button
//     PORTB 5 output for TV '6' button
//     PORTB 6 output for TV '7' button
//     PORTB 7 output for TV '8' button
//
// all outputs off for TV '0' button
// all outputs on  for TV '9' button

// works with 3.3V or 5V, but match PIC voltage to IR receiver chip voltage

// tested on EasyPIC v7 board at 3.3V with IR click board in mikroBUS socket 1
// fit PIC16F84A into DIP18B socket and jumper for 8MHz crystal. Turn on LEDs for PORTB
// Note:  You will need to change to 5Volt supply for programming

sbit ir_rx at PORTA.B2;                  // IR receiver 40KHz

// function prototypes
void get_mark_time(void);                // get Sony IR mark time


// global variables
unsigned char counter = 0;
unsigned char shadow = 0;
unsigned char bitcount;
unsigned char ir_address;
unsigned char ir_command;
unsigned int mark_time;

void interrupt() {

    if (INTCON.T0IF) {
        counter++;                       // increment counter
        INTCON.T0IF = 0;                 // Clear Timer0 overflow interrupt flag
    }
}

void main() {

    OPTION_REG = 0x04;             // TMR0 prescaler set to 1:32
    TRISA = 0x04;                  // IR b0 = input, rest output
    PORTA = 0;                     // all outputs off
    TRISB = 0;                     // all output
    PORTB = 0;                     // all low

    // start timer 0 counting
    INTCON.GIE = 1;               // Global interrupt enable
    INTCON.T0IE = 1;              // Enable Timer0 overflow interrupt
    
    while(1){
        ir_command = 0;                                        // initialise to prevent false trigger
        ir_address = 0;                                        // initialise to prevent false trigger
        get_mark_time();                                       // get Sony leader - 2.4mS Mark, 1.2mS space
        if ((mark_time > 0x80) && (mark_time < 0xb0)){         // expect about 0x98 - ignore anything but 2.4mS mark
             for(bitcount = 0 ; bitcount < 7 ; bitcount++){    // 7 bit command
                get_mark_time();                               // get a Sony IR bit
                ir_command >>= 1;                              // shift
                ir_command &= 0x7f;                            // top bit zero
                if (mark_time > 0x38){                         // > 38 is assumed to be a 1
                    ir_command ^= 0x80;                        // top bit 1
                }
            }
            ir_command >>= 1;                                  // shift 1 unused bit
            ir_command &= 0x7F;                                // clear top bit

            for(bitcount = 0 ; bitcount < 5 ; bitcount++){     // 5 bit address
                get_mark_time();                               // get a Sony IR bit
                ir_address >>= 1;                              // shift
                ir_address &= 0x7f;                            // top bit zero
                if (mark_time > 0x38){                         // > 38 is assumed to be a 1
                    ir_address ^= 0x80;                        // top bit 1
                }
            }
            ir_address >>= 3;                                  // shift 3 unused bits
            ir_address &= 0x1F;                                // clear top 3 bits

        }
        
        if(ir_address == 1){                                   // TV
            if(ir_command == 0){                               // button 1
                shadow ^= 0x01;                                // toggle output 0
                PORTB = shadow;
            }
            if(ir_command == 1){                               // button 2
                shadow ^= 0x02;                                // toggle output 1
                PORTB = shadow;
            }
            if(ir_command == 2){                               // button 3
                shadow ^= 0x04;                                // toggle output 2
                PORTB = shadow;
            }
            if(ir_command == 3){                               // button 4
                shadow ^= 0x08;                                // toggle output 3
                PORTB = shadow;
            }
            if(ir_command == 4){                               // button 5
                shadow ^= 0x10;                                // toggle output 4
                PORTB = shadow;
            }
            if(ir_command == 5){                               // button 6
                shadow ^= 0x20;                                // toggle output 5
                PORTB = shadow;
            }
            if(ir_command == 6){                               // button 7
                shadow ^= 0x40;                                // toggle output 6
                PORTB = shadow;
            }
            if(ir_command == 7){                               // button 8
                shadow ^= 0x80;                                // toggle output 7
                PORTB = shadow;
            }
            if(ir_command == 8){                               // button 9
                shadow = 0xff;                                 // all outputs on
                PORTB = shadow;
            }
            if(ir_command == 9){                               // button 0
                shadow = 0x00;                                 // all outputs off
                PORTB = shadow;
            }
            Delay_ms(200);                                      // avoid double toggle
        }
    }
}

// get time of mark, then ignore space
void get_mark_time(void){
    while(ir_rx);                           // wait for a mark
    counter=0;
    TMR0 = 0;
    while(!ir_rx);                          // wait for space
    mark_time = (counter << 8) + TMR0;      // collect integer mark time

}